@@ -635,6 +635,7 @@ src_tex2lyx_copied_files = Split('''
     ModuleList.cpp
     Spacing.cpp
     TextClass.cpp
+    version.cpp
     insets/InsetLayout.cpp
 ''')
 
@@ -1357,6 +1358,7 @@ lib_templates_files = Split('''
     agu_article.lyx
     apa.lyx
     beamer-conference-ornate-20min.lyx
+    ctex.lyx
     de_beamer-conference-ornate-20min.lyx
     dinbrief.lyx
     docbook_article.lyx
@@ -1381,11 +1383,35 @@ lib_templates_files = Split('''
     README.new_templates
     revtex4.lyx
     slides.lyx
+''')
+
+
+lib_templates_springer_files = Split('''
     svjour3.lyx
-    svmono_book.lyx
+    svmono_acknow.lyx
+    svmono_acronym.lyx
+    svmono_appendix.lyx
+    svmono_book-master.lyx
+    svmono_chapter.lyx
+    svmono_dedication.lyx
+    svmono_foreword.lyx
+    svmono_glossary.lyx
+    svmono_part.lyx
+    svmono_preface.lyx
+    svmono_referenc.lyx
+    svmono_solutions.lyx
+    svmult_acknow.lyx
+    svmult_acronym.lyx
     svmult_appendix.lyx
     svmult_author.lyx
-    svmult_editor.lyx
+    svmult_cblist.lyx
+    svmult_dedication.lyx
+    svmult_editor-master.lyx
+    svmult_foreword.lyx
+    svmult_glossary.lyx
+    svmult_part.lyx
+    svmult_preface.lyx
+    svmult_referenc.lyx
 ''')
 
 
@@ -1468,8 +1494,8 @@ lib_images_files = Split('''
     buffer-view_pdf4.png
     buffer-view_pdf5.png
     buffer-view_ps.png
-    buffer-write-as.png
     buffer-write.png
+    buffer-write-as.png
     build-program.png
     busy.gif
     change-accept.png
@@ -1506,7 +1532,6 @@ lib_images_files = Split('''
     float-insert_table.png
     font-bold.png
     font-emph.png
-    textstyle-apply.png
     font-noun.png
     font-sans.png
     footnote-insert.png
@@ -1587,6 +1612,7 @@ lib_images_files = Split('''
     tabular-feature_toggle-rotate-tabular.png
     tabular-feature_unset-all-lines.png
     tabular-insert.png
+    textstyle-apply.png
     thesaurus-entry.png
     toolbar-toggle_math.png
     toolbar-toggle_math_panels.png
@@ -2020,7 +2046,6 @@ lib_images_math_files = Split('''
     square.png
     star.png
     style.png
-    style.png
     sub.png
     subset.png
     subset2.png
@@ -2151,6 +2176,275 @@ lib_images_commands_files = Split('''
 ''')
 
 
+lib_images_classic_files = Split('''
+    all-changes-accept.png
+    all-changes-reject.png
+    bookmark-goto.png
+    bookmark-goto_0.png
+    bookmark-save.png
+    box-insert.png
+    break-line.png
+    buffer-close.png
+    buffer-export.png
+    buffer-export_dvi.png
+    buffer-export_dvi3.png
+    buffer-export_latex.png
+    buffer-export_pdf.png
+    buffer-export_pdf2.png
+    buffer-export_pdf3.png
+    buffer-export_pdf4.png
+    buffer-export_pdf5.png
+    buffer-export_ps.png
+    buffer-export_text.png
+    buffer-new.png
+    buffer-reload.png
+    buffer-toggle-output-sync.png
+    buffer-update.png
+    buffer-update_dvi.png
+    buffer-update_dvi3.png
+    buffer-update_pdf.png
+    buffer-update_pdf2.png
+    buffer-update_pdf3.png
+    buffer-update_pdf4.png
+    buffer-update_pdf5.png
+    buffer-update_ps.png
+    buffer-view.png
+    buffer-view_dvi.png
+    buffer-view_dvi3.png
+    buffer-view_pdf.png
+    buffer-view_pdf2.png
+    buffer-view_pdf3.png
+    buffer-view_pdf4.png
+    buffer-view_pdf5.png
+    buffer-view_ps.png
+    buffer-write.png
+    buffer-write-as.png
+    build-program.png
+    change-accept.png
+    change-next.png
+    change-reject.png
+    changes-merge.png
+    changes-output.png
+    changes-track.png
+    close-tab-group.png
+    copy.png
+    cut.png
+    demote.png
+    depth-decrement.png
+    depth-increment.png
+    dialog-preferences.png
+    dialog-show_character.png
+    dialog-show_findreplace.png
+    dialog-show_mathdelimiter.png
+    dialog-show_mathmatrix.png
+    dialog-show_print.png
+    dialog-show_spellchecker.png
+    dialog-show_vclog.png
+    dialog-show-new-inset_citation.png
+    dialog-show-new-inset_graphics.png
+    dialog-show-new-inset_include.png
+    dialog-show-new-inset_ref.png
+    dialog-toggle_findreplaceadv.png
+    dialog-toggle_toc.png
+    down.png
+    ert-insert.png
+    file-open.png
+    float-insert_figure.png
+    float-insert_table.png
+    font-bold.png
+    font-emph.png
+    font-noun.png
+    font-sans.png
+    footnote-insert.png
+    href-insert.png
+    iconsize.png
+    index-insert.png
+    info-insert_buffer_vcs-revision.png
+    label-insert.png
+    layout-document.png
+    layout-paragraph.png
+    layout.png
+    layout_Description.png
+    layout_Enumerate.png
+    layout_Itemize.png
+    layout_List.png
+    layout_LyX-Code.png
+    layout_Scrap.png
+    layout_Section.png
+    marginalnote-insert.png
+    master-buffer-update.png
+    master-buffer-view.png
+    math-display.png
+    math-macro-add-greedy-optional-param.png
+    math-macro-add-optional-param.png
+    math-macro-add-param.png
+    math-macro-append-greedy-param.png
+    math-macro-make-nonoptional.png
+    math-macro-make-optional.png
+    math-macro-remove-greedy-param.png
+    math-macro-remove-optional-param.png
+    math-macro-remove-param.png
+    math-macro_newmacroname_newcommand.png
+    math-matrix.png
+    math-mode.png
+    math-subscript.png
+    math-superscript.png
+    nomencl-insert.png
+    note-insert.png
+    note-next.png
+    paste.png
+    promote.png
+    redo.png
+    reload.png
+    script-insert_subscript.png
+    script-insert_superscript.png
+    split-view_horizontal.png
+    split-view_vertical.png
+    tabular-feature_align-decimal.png
+    tabular-feature_append-column.png
+    tabular-feature_append-row.png
+    tabular-feature_delete-column.png
+    tabular-feature_delete-row.png
+    tabular-feature_m-align-center.png
+    tabular-feature_m-align-left.png
+    tabular-feature_m-align-right.png
+    tabular-feature_m-valign-bottom.png
+    tabular-feature_m-valign-middle.png
+    tabular-feature_m-valign-top.png
+    tabular-feature_multicolumn.png
+    tabular-feature_multirow.png
+    tabular-feature_set-all-lines.png
+    tabular-feature_set-border-lines.png
+    tabular-feature_set-longtabular.png
+    tabular-feature_set-rotate-cell.png
+    tabular-feature_set-rotate-tabular.png
+    tabular-feature_toggle-line-bottom.png
+    tabular-feature_toggle-line-left.png
+    tabular-feature_toggle-line-right.png
+    tabular-feature_toggle-line-top.png
+    tabular-feature_toggle-rotate-cell.png
+    tabular-feature_toggle-rotate-tabular.png
+    tabular-feature_unset-all-lines.png
+    tabular-insert.png
+    textstyle-apply.png
+    thesaurus-entry.png
+    toolbar-toggle_math.png
+    toolbar-toggle_math_panels.png
+    toolbar-toggle_table.png
+    undo.png
+    unknown.png
+    up.png
+    update-others.png
+    url-insert.png
+    vc-compare.png
+    vc-compare_0.png
+    vc-check-in.png
+    vc-check-out.png
+    vc-locking-toggle.png
+    vc-register.png
+    vc-repo-update.png
+    vc-revert.png
+    view-others.png
+''')
+
+
+lib_images_oxygen_files = Split('''
+    all-changes-accept.png
+    all-changes-reject.png
+    bookmark-goto.png
+    bookmark-goto_0.png
+    bookmark-save.png
+    box-insert.png
+    buffer-new.png
+    buffer-reload.png
+    buffer-toggle-output-sync.png
+    buffer-update.png
+    buffer-update_dvi.png
+    buffer-update_dvi3.png
+    buffer-update_pdf.png
+    buffer-update_pdf2.png
+    buffer-update_pdf3.png
+    buffer-update_pdf4.png
+    buffer-update_pdf5.png
+    buffer-update_ps.png
+    buffer-view.png
+    buffer-view_dvi.png
+    buffer-view_dvi3.png
+    buffer-view_pdf.png
+    buffer-view_pdf2.png
+    buffer-view_pdf3.png
+    buffer-view_pdf4.png
+    buffer-view_pdf5.png
+    buffer-view_ps.png
+    buffer-write.png
+    buffer-write-as.png
+    change-accept.png
+    change-next.png
+    change-reject.png
+    changes-output.png
+    changes-track.png
+    closetab.png
+    close-tab-group.png
+    copy.png
+    cut.png
+    depth-decrement.png
+    depth-increment.png
+    dialog-show_character.png
+    dialog-show_findreplace.png
+    dialog-show_print.png
+    dialog-show_spellchecker.png
+    dialog-show-new-inset_citation.png
+    dialog-show-new-inset_graphics.png
+    dialog-show-new-inset_include.png
+    dialog-show-new-inset_ref.png
+    dialog-toggle_findreplaceadv.png
+    dialog-toggle_toc.png
+    ert-insert.png
+    file-open.png
+    float-insert_figure.png
+    float-insert_table.png
+    font-bold.png
+    font-emph.png
+    font-noun.png
+    footnote-insert.png
+    href-insert.png
+    iconsize.png
+    index-insert.png
+    label-insert.png
+    layout.png
+    layout-document.png
+    layout-paragraph.png
+    layout_Description.png
+    layout_Enumerate.png
+    layout_Itemize.png
+    layout_List.png
+    lyx-quit.png
+    marginalnote-insert.png
+    master-buffer-update.png
+    master-buffer-view.png
+    math-macro_newmacroname_newcommand.png
+    math-mode.png
+    nomencl-insert.png
+    note-insert.png
+    note-next.png
+    paste.png
+    redo.png
+    reload.png
+    split-view_horizontal.png
+    split-view_vertical.png
+    tabular-insert.png
+    textstyle-apply.png
+    thesaurus-entry.png
+    toolbar-toggle_math.png
+    toolbar-toggle_table.png
+    undo.png
+    update-others.png
+    url-insert.png
+    vc-revert.png
+    view-others.png
+''')
+
+
 lib_images_attic_extra_files = Split('''
     dialog-show_mathpanel.png
 ''')
@@ -2343,6 +2637,7 @@ lib_doc_fr_clipart_files = Split('''
     BoxInsetDefaultQt4.png
     ChildDocumentQt4.png
     CommentNoteImageQt4.png
+    ERT.png
     floatQt4.png
     footnoteQt4.png
     GreyedOutNoteImageQt4.png
@@ -2538,6 +2833,7 @@ lib_examples_files = Split('''
     listerrors.lyx
     Literate.lyx
     localization_test.lyx
+    longsheet.gnumeric
     modernCV.lyx
     multicol.lyx
     noweb2lyx.lyx
@@ -2548,8 +2844,12 @@ lib_examples_files = Split('''
     serial_letter1.lyx
     serial_letter2.lyx
     serial_letter3.lyx
+    sheet1.gnumeric
+    sheet2.ods
+    sheet3.xls
     simplecv.lyx
     splash.lyx
+    spreadsheet.lyx
     sweave.lyx
     tufte-book.lyx
     tufte-handout.lyx
@@ -2799,6 +3099,9 @@ lib_layouts_files = Split('''
     broadway.layout
     chess.layout
     cl2emult.layout
+    ctex-article.layout
+    ctex-book.layout
+    ctex-report.layout
     customHeadersFooters.module
     dinbrief.layout
     docbook-book.layout
@@ -2926,7 +3229,7 @@ lib_layouts_inc_files = Split('''
     stdstarsections.inc
     stdstruct.inc
     stdtitle.inc
-    svjour.inc
+    svcommon.inc
     theorems.inc
     theorems-ams.inc
     theorems-ams-bytype.inc
@@ -2937,6 +3240,7 @@ lib_layouts_inc_files = Split('''
     theorems-refprefix.inc
     theorems-starred.inc
     theorems-starred-equivalents.inc
+    theorems-without-preamble.inc
 ''')
 
 
@@ -2991,6 +3295,7 @@ lib_scripts_files = Split('''
     lyxpreview-lytex2bitmap.py
     lyxpreview-platex2bitmap.py
     lyxpreview_tools.py
+    lyxstangle.R
     lyxsweave.R
     prefs2prefs.py
     prefs2prefs_lfuns.py
