@@ -75,7 +75,7 @@
 /* Copy the first part of user declarations.  */
 
 /* Line 189 of yacc.c  */
-#line 1 "/cvs/lyx-export/intl/plural.y"
+#line 1 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
 
 /* Expression parsing for plural form selection.
    Copyright (C) 2000-2001, 2003, 2005 Free Software Foundation, Inc.
@@ -125,7 +125,7 @@
 
 
 /* Line 189 of yacc.c  */
-#line 129 "/cvs/lyx-export/intl/plural.c"
+#line 129 "/cvs/lyx/lyx-release/lyx-export/intl/plural.c"
 
 /* Enabling traces.  */
 #ifndef YYDEBUG
@@ -174,7 +174,7 @@ typedef union YYSTYPE
 {
 
 /* Line 214 of yacc.c  */
-#line 51 "/cvs/lyx-export/intl/plural.y"
+#line 51 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
 
   unsigned long int num;
   enum operator op;
@@ -183,7 +183,7 @@ typedef union YYSTYPE
 
 
 /* Line 214 of yacc.c  */
-#line 187 "/cvs/lyx-export/intl/plural.c"
+#line 187 "/cvs/lyx/lyx-release/lyx-export/intl/plural.c"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
@@ -194,7 +194,7 @@ typedef union YYSTYPE
 /* Copy the second part of user declarations.  */
 
 /* Line 264 of yacc.c  */
-#line 57 "/cvs/lyx-export/intl/plural.y"
+#line 57 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
 
 /* Prototypes for local functions.  */
 static int yylex (YYSTYPE *lval, const char **pexp);
@@ -271,7 +271,7 @@ new_exp_3 (enum operator op, struct expression *bexp,
 
 
 /* Line 264 of yacc.c  */
-#line 275 "/cvs/lyx-export/intl/plural.c"
+#line 275 "/cvs/lyx/lyx-release/lyx-export/intl/plural.c"
 
 #ifdef short
 # undef short
@@ -1483,7 +1483,7 @@ yyreduce:
         case 2:
 
 /* Line 1464 of yacc.c  */
-#line 153 "/cvs/lyx-export/intl/plural.y"
+#line 153 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
     {
 	    if ((yyvsp[(1) - (1)].exp) == NULL)
 	      YYABORT;
@@ -1494,7 +1494,7 @@ yyreduce:
   case 3:
 
 /* Line 1464 of yacc.c  */
-#line 161 "/cvs/lyx-export/intl/plural.y"
+#line 161 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
     {
 	    (yyval.exp) = new_exp_3 (qmop, (yyvsp[(1) - (5)].exp), (yyvsp[(3) - (5)].exp), (yyvsp[(5) - (5)].exp));
 	  }
@@ -1503,7 +1503,7 @@ yyreduce:
   case 4:
 
 /* Line 1464 of yacc.c  */
-#line 165 "/cvs/lyx-export/intl/plural.y"
+#line 165 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
     {
 	    (yyval.exp) = new_exp_2 (lor, (yyvsp[(1) - (3)].exp), (yyvsp[(3) - (3)].exp));
 	  }
@@ -1512,7 +1512,7 @@ yyreduce:
   case 5:
 
 /* Line 1464 of yacc.c  */
-#line 169 "/cvs/lyx-export/intl/plural.y"
+#line 169 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
     {
 	    (yyval.exp) = new_exp_2 (land, (yyvsp[(1) - (3)].exp), (yyvsp[(3) - (3)].exp));
 	  }
@@ -1521,7 +1521,7 @@ yyreduce:
   case 6:
 
 /* Line 1464 of yacc.c  */
-#line 173 "/cvs/lyx-export/intl/plural.y"
+#line 173 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
     {
 	    (yyval.exp) = new_exp_2 ((yyvsp[(2) - (3)].op), (yyvsp[(1) - (3)].exp), (yyvsp[(3) - (3)].exp));
 	  }
@@ -1530,7 +1530,7 @@ yyreduce:
   case 7:
 
 /* Line 1464 of yacc.c  */
-#line 177 "/cvs/lyx-export/intl/plural.y"
+#line 177 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
     {
 	    (yyval.exp) = new_exp_2 ((yyvsp[(2) - (3)].op), (yyvsp[(1) - (3)].exp), (yyvsp[(3) - (3)].exp));
 	  }
@@ -1539,7 +1539,7 @@ yyreduce:
   case 8:
 
 /* Line 1464 of yacc.c  */
-#line 181 "/cvs/lyx-export/intl/plural.y"
+#line 181 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
     {
 	    (yyval.exp) = new_exp_2 ((yyvsp[(2) - (3)].op), (yyvsp[(1) - (3)].exp), (yyvsp[(3) - (3)].exp));
 	  }
@@ -1548,7 +1548,7 @@ yyreduce:
   case 9:
 
 /* Line 1464 of yacc.c  */
-#line 185 "/cvs/lyx-export/intl/plural.y"
+#line 185 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
     {
 	    (yyval.exp) = new_exp_2 ((yyvsp[(2) - (3)].op), (yyvsp[(1) - (3)].exp), (yyvsp[(3) - (3)].exp));
 	  }
@@ -1557,7 +1557,7 @@ yyreduce:
   case 10:
 
 /* Line 1464 of yacc.c  */
-#line 189 "/cvs/lyx-export/intl/plural.y"
+#line 189 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
     {
 	    (yyval.exp) = new_exp_1 (lnot, (yyvsp[(2) - (2)].exp));
 	  }
@@ -1566,7 +1566,7 @@ yyreduce:
   case 11:
 
 /* Line 1464 of yacc.c  */
-#line 193 "/cvs/lyx-export/intl/plural.y"
+#line 193 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
     {
 	    (yyval.exp) = new_exp_0 (var);
 	  }
@@ -1575,7 +1575,7 @@ yyreduce:
   case 12:
 
 /* Line 1464 of yacc.c  */
-#line 197 "/cvs/lyx-export/intl/plural.y"
+#line 197 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
     {
 	    if (((yyval.exp) = new_exp_0 (num)) != NULL)
 	      (yyval.exp)->val.num = (yyvsp[(1) - (1)].num);
@@ -1585,7 +1585,7 @@ yyreduce:
   case 13:
 
 /* Line 1464 of yacc.c  */
-#line 202 "/cvs/lyx-export/intl/plural.y"
+#line 202 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
     {
 	    (yyval.exp) = (yyvsp[(2) - (3)].exp);
 	  }
@@ -1594,7 +1594,7 @@ yyreduce:
 
 
 /* Line 1464 of yacc.c  */
-#line 1598 "/cvs/lyx-export/intl/plural.c"
+#line 1598 "/cvs/lyx/lyx-release/lyx-export/intl/plural.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -1806,7 +1806,7 @@ yyreturn:
 
 
 /* Line 1684 of yacc.c  */
-#line 207 "/cvs/lyx-export/intl/plural.y"
+#line 207 "/cvs/lyx/lyx-release/lyx-export/intl/plural.y"
 
 
 void
