@@ -32,7 +32,7 @@ try:
     import lyx2lyx_version
     version__ = lyx2lyx_version.version
 except: # we are running from build directory so assume the last version
-    version__ = '1.6.0svn'
+    version__ = '2.0'
 
 default_debug__ = 2
 
@@ -73,14 +73,15 @@ format_relation = [("0_06",    [200], minor_versions("0.6" , 4)),
                    ("0_12",    [215], minor_versions("0.12", 1) + ["0.11"]),
                    ("1_0",     [215], minor_versions("1.0" , 4)),
                    ("1_1",     [215], minor_versions("1.1" , 4)),
-                   ("1_1_5",   [216], ["1.1.5","1.1.5.1","1.1.5.2","1.1"]),
-                   ("1_1_6_0", [217], ["1.1.6","1.1.6.1","1.1.6.2","1.1"]),
-                   ("1_1_6_3", [218], ["1.1.6.3","1.1.6.4","1.1"]),
+                   ("1_1_5",   [216], ["1.1", "1.1.5","1.1.5.1","1.1.5.2"]),
+                   ("1_1_6_0", [217], ["1.1", "1.1.6","1.1.6.1","1.1.6.2"]),
+                   ("1_1_6_3", [218], ["1.1", "1.1.6.3","1.1.6.4"]),
                    ("1_2",     [220], minor_versions("1.2" , 4)),
                    ("1_3",     [221], minor_versions("1.3" , 7)),
                    ("1_4", range(222,246), minor_versions("1.4" , 5)),
-                   ("1_5", range(246,277), minor_versions("1.5" , 6)),
-                   ("1_6", range(277,346), minor_versions("1.6" , 0))]
+                   ("1_5", range(246,277), minor_versions("1.5" , 7)),
+                   ("1_6", range(277,346), minor_versions("1.6" , 10)),
+                   ("2_0", range(346,414), minor_versions("2.0" , 0))]
 
 ####################################################################
 # This is useful just for development versions                     #
@@ -104,6 +105,23 @@ def formats_list():
     return formats
 
 
+def format_info():
+    " Returns a list with supported file formats."
+    out = """Major version:
+	minor versions
+	formats
+"""
+    for version in format_relation:
+        major = str(version[2][0])
+        versions = str(version[2][1:])
+        if len(version[1]) == 1:
+            formats = str(version[1][0])
+        else:
+            formats = "%s - %s" % (version[1][-1], version[1][0])
+        out += "%s\n\t%s\n\t%s\n\n" % (major, versions, formats)
+    return out + '\n'
+
+
 def get_end_format():
     " Returns the more recent file format available."
     return format_relation[-1][1][-1]
@@ -158,7 +176,7 @@ class LyX_base:
 
     def __init__(self, end_format = 0, input = "", output = "", error = "",
                  debug = default_debug__, try_hard = 0, cjk_encoding = '',
-                 language = "english", encoding = "auto"):
+                 final_version = "", language = "english", encoding = "auto"):
 
         """Arguments:
         end_format: final format that the file should be converted. (integer)
@@ -180,9 +198,37 @@ class LyX_base:
 
         if end_format:
             self.end_format = self.lyxformat(end_format)
+
+            # In case the target version and format are both specified
+            # verify that they are compatible. If not send a warning
+            # and ignore the version.
+            if final_version:
+                message = "Incompatible version %s for specified format %d" % (
+                    final_version, self.end_format)
+                for version in format_relation:
+                    if self.end_format in version[1]:
+                        if final_version not in version[2]:
+                            self.warning(message)
+                            final_version = ""
+        elif final_version:
+            for version in format_relation:
+                if final_version in version[2]:
+                    # set the last format for that version
+                    self.end_format = version[1][-1]
+                    break
+            else:
+                final_version = ""
         else:
             self.end_format = get_end_format()
 
+        if not final_version:
+            for step in format_relation:
+                if self.end_format in step[1]:
+                    final_version = step[2][1]
+        self.final_version = final_version
+        self.warning("Final version: %s" % self.final_version, 10)
+        self.warning("Final format: %d" % self.end_format, 10)
+
         self.backend = "latex"
         self.textclass = "article"
         # This is a hack: We use '' since we don't know the default
@@ -208,7 +254,7 @@ class LyX_base:
         " Emits a warning and exits if not in try_hard mode."
         self.warning(message)
         if not self.try_hard:
-            self.warning("Quiting.")
+            self.warning("Quitting.")
             sys.exit(1)
 
         self.status = 2
@@ -475,16 +521,16 @@ class LyX_base:
 
     def convert(self):
         "Convert from current (self.format) to self.end_format."
-        mode, convertion_chain = self.chain()
-        self.warning("convertion chain: " + str(convertion_chain), 3)
+        mode, conversion_chain = self.chain()
+        self.warning("conversion chain: " + str(conversion_chain), 3)
 
-        for step in convertion_chain:
+        for step in conversion_chain:
             steps = getattr(__import__("lyx_" + step), mode)
 
             self.warning("Convertion step: %s - %s" % (step, mode),
                          default_debug__ + 1)
             if not steps:
-                self.error("The convertion to an older "
+                self.error("The conversion to an older "
                 "format (%s) is not implemented." % self.format)
 
             multi_conv = len(steps) != 1
@@ -517,7 +563,7 @@ class LyX_base:
 
     def chain(self):
         """ This is where all the decisions related with the
-        convertion are taken.  It returns a list of modules needed to
+        conversion are taken.  It returns a list of modules needed to
         convert the LyX file from self.format to self.end_format"""
 
         self.start =  self.format
@@ -556,13 +602,14 @@ class LyX_base:
         steps = []
         if (initial_step, self.start) < (final_step, self.end_format):
             mode = "convert"
-            first_step = 1
+            full_steps = []
             for step in format_relation:
-                if  initial_step <= step[0] <= final_step:
-                    if first_step and len(step[1]) == 1:
-                        first_step = 0
-                        continue
-                    steps.append(step[0])
+                if  initial_step <= step[0] <= final_step and step[2][0] <= self.final_version:
+                    full_steps.append(step)
+            if full_steps[0][1][-1] == self.format:
+                full_steps = full_steps[1:]
+            for step in full_steps:
+                steps.append(step[0])
         else:
             mode = "revert"
             relation_format = format_relation[:]
@@ -581,162 +628,167 @@ class LyX_base:
         return mode, steps
 
 
-    def get_toc(self, depth = 4):
-        " Returns the TOC of this LyX document."
-        paragraphs_filter = {'Title' : 0,'Chapter' : 1, 'Section' : 2,
-                             'Subsection' : 3, 'Subsubsection': 4}
-        allowed_insets = ['Quotes']
-        allowed_parameters = ('\\paragraph_spacing', '\\noindent',
-                              '\\align', '\\labelwidthstring',
-                              "\\start_of_appendix", "\\leftindent")
-        sections = []
-        for section in paragraphs_filter.keys():
-            sections.append('\\begin_layout %s' % section)
+# Part of an unfinished attempt to make lyx2lyx gave a more
+# structured view of the document.
+#    def get_toc(self, depth = 4):
+#        " Returns the TOC of this LyX document."
+#        paragraphs_filter = {'Title' : 0,'Chapter' : 1, 'Section' : 2,
+#                             'Subsection' : 3, 'Subsubsection': 4}
+#        allowed_insets = ['Quotes']
+#        allowed_parameters = ('\\paragraph_spacing', '\\noindent',
+#                              '\\align', '\\labelwidthstring',
+#                              "\\start_of_appendix", "\\leftindent")
+#        sections = []
+#        for section in paragraphs_filter.keys():
+#            sections.append('\\begin_layout %s' % section)
 
-        toc_par = []
-        i = 0
-        while 1:
-            i = find_tokens(self.body, sections, i)
-            if i == -1:
-                break
+#        toc_par = []
+#        i = 0
+#        while 1:
+#            i = find_tokens(self.body, sections, i)
+#            if i == -1:
+#                break
 
-            j = find_end_of(self.body,  i + 1, '\\begin_layout', '\\end_layout')
-            if j == -1:
-                self.warning('Incomplete file.', 0)
-                break
+#            j = find_end_of(self.body,  i + 1, '\\begin_layout', '\\end_layout')
+#            if j == -1:
+#                self.warning('Incomplete file.', 0)
+#                break
 
-            section = self.body[i].split()[1]
-            if section[-1] == '*':
-                section = section[:-1]
+#            section = self.body[i].split()[1]
+#            if section[-1] == '*':
+#                section = section[:-1]
 
-            par = []
+#            par = []
 
-            k = i + 1
-            # skip paragraph parameters
-            while not self.body[k].strip() or self.body[k].split()[0] \
-                      in allowed_parameters:
-                k += 1
+#            k = i + 1
+#            # skip paragraph parameters
+#            while not self.body[k].strip() or self.body[k].split()[0] \
+#                      in allowed_parameters:
+#                k += 1
 
-            while k < j:
-                if check_token(self.body[k], '\\begin_inset'):
-                    inset = self.body[k].split()[1]
-                    end = find_end_of_inset(self.body, k)
-                    if end == -1 or end > j:
-                        self.warning('Malformed file.', 0)
+#            while k < j:
+#                if check_token(self.body[k], '\\begin_inset'):
+#                    inset = self.body[k].split()[1]
+#                    end = find_end_of_inset(self.body, k)
+#                    if end == -1 or end > j:
+#                        self.warning('Malformed file.', 0)
 
-                    if inset in allowed_insets:
-                        par.extend(self.body[k: end+1])
-                    k = end + 1
-                else:
-                    par.append(self.body[k])
-                    k += 1
+#                    if inset in allowed_insets:
+#                        par.extend(self.body[k: end+1])
+#                    k = end + 1
+#                else:
+#                    par.append(self.body[k])
+#                    k += 1
 
-            # trim empty lines in the end.
-            while par and par[-1].strip() == '':
-                par.pop()
+#            # trim empty lines in the end.
+#            while par and par[-1].strip() == '':
+#                par.pop()
 
-            toc_par.append(Paragraph(section, par))
+#            toc_par.append(Paragraph(section, par))
 
-            i = j + 1
+#            i = j + 1
 
-        return toc_par
+#        return toc_par
 
 
 class File(LyX_base):
     " This class reads existing LyX files."
 
     def __init__(self, end_format = 0, input = "", output = "", error = "",
-                 debug = default_debug__, try_hard = 0, cjk_encoding = ''):
+                 debug = default_debug__, try_hard = 0, cjk_encoding = '',
+                 final_version = ''):
         LyX_base.__init__(self, end_format, input, output, error,
-                          debug, try_hard, cjk_encoding)
+                          debug, try_hard, cjk_encoding, final_version)
         self.read()
 
 
-class NewFile(LyX_base):
-    " This class is to create new LyX files."
-    def set_header(self, **params):
-        # set default values
-        self.header.extend([
-            "#LyX xxxx created this file."
-            "For more info see http://www.lyx.org/",
-            "\\lyxformat xxx",
-            "\\begin_document",
-            "\\begin_header",
-            "\\textclass article",
-            "\\language english",
-            "\\inputencoding auto",
-            "\\font_roman default",
-            "\\font_sans default",
-            "\\font_typewriter default",
-            "\\font_default_family default",
-            "\\font_sc false",
-            "\\font_osf false",
-            "\\font_sf_scale 100",
-            "\\font_tt_scale 100",
-            "\\graphics default",
-            "\\paperfontsize default",
-            "\\papersize default",
-            "\\use_geometry false",
-            "\\use_amsmath 1",
-            "\\cite_engine basic",
-            "\\use_bibtopic false",
-            "\\paperorientation portrait",
-            "\\secnumdepth 3",
-            "\\tocdepth 3",
-            "\\paragraph_separation indent",
-            "\\defskip medskip",
-            "\\quotes_language english",
-            "\\papercolumns 1",
-            "\\papersides 1",
-            "\\paperpagestyle default",
-            "\\tracking_changes false",
-            "\\end_header"])
+#class NewFile(LyX_base):
+#    " This class is to create new LyX files."
+#    def set_header(self, **params):
+#        # set default values
+#        self.header.extend([
+#            "#LyX xxxx created this file."
+#            "For more info see http://www.lyx.org/",
+#            "\\lyxformat xxx",
+#            "\\begin_document",
+#            "\\begin_header",
+#            "\\textclass article",
+#            "\\language english",
+#            "\\inputencoding auto",
+#            "\\font_roman default",
+#            "\\font_sans default",
+#            "\\font_typewriter default",
+#            "\\font_default_family default",
+#            "\\font_sc false",
+#            "\\font_osf false",
+#            "\\font_sf_scale 100",
+#            "\\font_tt_scale 100",
+#            "\\graphics default",
+#            "\\paperfontsize default",
+#            "\\papersize default",
+#            "\\use_geometry false",
+#            "\\use_amsmath 1",
+#            "\\cite_engine basic",
+#            "\\use_bibtopic false",
+#            "\\paperorientation portrait",
+#            "\\secnumdepth 3",
+#            "\\tocdepth 3",
+#            "\\paragraph_separation indent",
+#            "\\defskip medskip",
+#            "\\quotes_language english",
+#            "\\papercolumns 1",
+#            "\\papersides 1",
+#            "\\paperpagestyle default",
+#            "\\tracking_changes false",
+#            "\\end_header"])
 
-        self.format = get_end_format()
-        for param in params:
-            self.set_parameter(param, params[param])
+#        self.format = get_end_format()
+#        for param in params:
+#            self.set_parameter(param, params[param])
 
 
-    def set_body(self, paragraphs):
-        self.body.extend(['\\begin_body',''])
+#    def set_body(self, paragraphs):
+#        self.body.extend(['\\begin_body',''])
 
-        for par in paragraphs:
-            self.body.extend(par.asLines())
+#        for par in paragraphs:
+#            self.body.extend(par.asLines())
 
-        self.body.extend(['','\\end_body', '\\end_document'])
+#        self.body.extend(['','\\end_body', '\\end_document'])
 
 
-class Paragraph:
-    # unfinished implementation, it is missing the Text and Insets
-    # representation.
-    " This class represents the LyX paragraphs."
-    def __init__(self, name, body=[], settings = [], child = []):
-        """ Parameters:
-        name: paragraph name.
-        body: list of lines of body text.
-        child: list of paragraphs that descend from this paragraph.
-        """
-        self.name = name
-        self.body = body
-        self.settings = settings
-        self.child = child
+# Part of an unfinished attempt to make lyx2lyx gave a more
+# structured view of the document.
+#class Paragraph:
+#    # unfinished implementation, it is missing the Text and Insets
+#    # representation.
+#    " This class represents the LyX paragraphs."
+#    def __init__(self, name, body=[], settings = [], child = []):
+#        """ Parameters:
+#        name: paragraph name.
+#        body: list of lines of body text.
+#        child: list of paragraphs that descend from this paragraph.
+#        """
+#        self.name = name
+#        self.body = body
+#        self.settings = settings
+#        self.child = child
 
-    def asLines(self):
-        """ Converts the paragraph to a list of strings, representing
-        it in the LyX file."""
+#    def asLines(self):
+#        """ Converts the paragraph to a list of strings, representing
+#        it in the LyX file."""
 
-        result = ['','\\begin_layout %s' % self.name]
-        result.extend(self.settings)
-        result.append('')
-        result.extend(self.body)
-        result.append('\\end_layout')
+#        result = ['','\\begin_layout %s' % self.name]
+#        result.extend(self.settings)
+#        result.append('')
+#        result.extend(self.body)
+#        result.append('\\end_layout')
 
-        if not self.child:
-            return result
+#        if not self.child:
+#            return result
 
-        result.append('\\begin_deeper')
-        for node in self.child:
-            result.extend(node.asLines())
-        result.append('\\end_deeper')
+#        result.append('\\begin_deeper')
+#        for node in self.child:
+#            result.extend(node.asLines())
+#        result.append('\\end_deeper')
 
-        return result
+#        return result
