@@ -75,6 +75,8 @@ class FileNameList;
 
 class Buffer;
 typedef std::list<Buffer *> ListOfBuffers;
+/// a list of Buffers we cloned
+typedef std::set<Buffer *> CloneList;
 
 
 /** The buffer object.
@@ -211,6 +213,10 @@ public:
 	int readHeader(Lexer & lex);
 
 private:
+	///
+	typedef std::map<Buffer const *, Buffer *> BufferMap;
+	///
+	void clone(BufferMap &, CloneList *) const;
 	/// save timestamp and checksum of the given file.
 	void saveCheckSum() const;	
 	/// read a new file
@@ -268,6 +274,7 @@ public:
 
 	/// Just a wrapper for writeLaTeXSource, first creating the ofstream.
 	bool makeLaTeXFile(support::FileName const & filename,
+			   std::string const & original_path,
 			   OutputParams const &,
 			   bool output_preamble = true,
 			   bool output_body = true) const;
@@ -294,6 +301,7 @@ public:
 	    \endcode
 	 */
 	void writeLaTeXSource(otexstream & os,
+			   std::string const & original_path,
 			   OutputParams const &,
 			   bool output_preamble = true,
 			   bool output_body = true) const;
