@@ -266,6 +266,8 @@ OutputParams::FLAVOR Converters::getFlavor(Graph::EdgePath const & path)
 				return OutputParams::XETEX;
 			if (conv.latex_flavor == "lualatex")
 				return OutputParams::LUATEX;
+			if (conv.latex_flavor == "dvilualatex")
+				return OutputParams::DVILUATEX;
 			if (conv.latex_flavor == "pdflatex")
 				return OutputParams::PDFLATEX;
 		if (conv.xml)
@@ -307,7 +309,8 @@ bool Converters::convert(Buffer const * buffer,
 			LYXERR(Debug::FILES, "No converter defined! "
 				   "I use convertDefault.py:\n\t" << command);
 			Systemcall one;
-			one.startscript(Systemcall::Wait, command);
+			one.startscript(Systemcall::Wait, command, buffer ?
+					buffer->filePath() : string());
 			if (to_file.isReadableFile()) {
 				if (conversionflags & try_cache)
 					ConverterCache::get().add(orig_from,
@@ -337,7 +340,7 @@ bool Converters::convert(Buffer const * buffer,
 	runparams.flavor = getFlavor(edgepath);
 
 	if (buffer) {
-		runparams.use_japanese = buffer->bufferFormat() == "platex";
+		runparams.use_japanese = buffer->params().bufferFormat() == "platex";
 		runparams.use_indices = buffer->params().use_indices;
 		runparams.bibtex_command = (buffer->params().bibtex_command == "default") ?
 			string() : buffer->params().bibtex_command;
@@ -443,12 +446,15 @@ bool Converters::convert(Buffer const * buffer,
 			int res;
 			if (dummy) {
 				res = one.startscript(Systemcall::DontWait,
-					to_filesystem8bit(from_utf8(command)));
+					to_filesystem8bit(from_utf8(command)),
+					buffer ? buffer->filePath() : string());
 				// We're not waiting for the result, so we can't do anything
 				// else here.
 			} else {
 				res = one.startscript(Systemcall::Wait,
-						to_filesystem8bit(from_utf8(command)));
+						to_filesystem8bit(from_utf8(command)),
+						buffer ? buffer->filePath()
+						       : string());
 				if (!real_outfile.empty()) {
 					Mover const & mover = getMover(conv.to);
 					if (!mover.rename(outfile, real_outfile))
@@ -468,7 +474,8 @@ bool Converters::convert(Buffer const * buffer,
 						" < " + quoteName(infile2 + ".out") +
 						" > " + quoteName(logfile);
 					one.startscript(Systemcall::Wait,
-						to_filesystem8bit(from_utf8(command2)));
+						to_filesystem8bit(from_utf8(command2)),
+						buffer->filePath());
 					if (!scanLog(*buffer, command, makeAbsPath(logfile, path), errorList))
 						return false;
 				}
@@ -612,7 +619,8 @@ bool Converters::runLaTeX(Buffer const & buffer, string const & command,
 
 	// do the LaTeX run(s)
 	string const name = buffer.latexName();
-	LaTeX latex(command, runparams, FileName(makeAbsPath(name)));
+	LaTeX latex(command, runparams, FileName(makeAbsPath(name)),
+		    buffer.filePath());
 	TeXErrors terr;
 	ShowMessage show(buffer);
 	latex.message.connect(show);
@@ -770,6 +778,7 @@ vector<string> Converters::savers() const
 	v.push_back("latex");
 	v.push_back("literate");
 	v.push_back("luatex");
+	v.push_back("dviluatex");
 	v.push_back("lyx");
 	v.push_back("xhtml");
 	v.push_back("pdflatex");
