@@ -1159,11 +1159,24 @@ docstring const LaTeXFeatures::getTClassHTMLStyles() const {
 
 
 namespace {
-docstring const getFloatI18nPreamble(docstring const & type, docstring const & name, docstring const & lang)
+docstring const getFloatI18nPreamble(docstring const & type,
+			docstring const & name, Language const * lang,
+			Encoding const & enc, bool const polyglossia)
 {
+	docstring const language = polyglossia ? from_ascii(lang->polyglossia())
+					       : from_ascii(lang->babel());
+	docstring const langenc = from_ascii(lang->encoding()->iconvName());
+	docstring const texenc = from_ascii(lang->encoding()->latexName());
+	docstring const bufenc = from_ascii(enc.iconvName());
+	docstring const s1 = docstring(1, 0xF0000);
+	docstring const s2 = docstring(1, 0xF0001);
+	docstring const translated = (langenc == bufenc) ? name
+		: from_ascii("\\inputencoding{") + texenc + from_ascii("}")
+			+ s1 + langenc + s2 + translated + s1 + bufenc + s2;
+
 	odocstringstream os;
-	os << "\\addto\\captions" << lang
-	   << "{\\renewcommand{\\" << type << "name}{" << name << "}}\n";
+	os << "\\addto\\captions" << language
+	   << "{\\renewcommand{\\" << type << "name}{" << translated << "}}\n";
 	return os.str();
 }
 }
@@ -1183,13 +1196,19 @@ docstring const LaTeXFeatures::getTClassI18nPreamble(bool use_babel, bool use_po
 	for (; cit != end; ++cit) {
 		// language dependent commands (once per document)
 		snippets.insert(tclass[*cit].langpreamble(buffer().language(),
-							  use_polyglossia));
+						buffer().params().encoding(),
+						use_polyglossia));
 		// commands for language changing (for multilanguage documents)
 		if ((use_babel || use_polyglossia) && !UsedLanguages_.empty()) {
-			snippets.insert(tclass[*cit].babelpreamble(buffer().language(),
-								   use_polyglossia));
+			snippets.insert(tclass[*cit].babelpreamble(
+						buffer().language(),
+						buffer().params().encoding(),
+						use_polyglossia));
 			for (lang_it lit = lbeg; lit != lend; ++lit)
-				snippets.insert(tclass[*cit].babelpreamble(*lit, use_polyglossia));
+				snippets.insert(tclass[*cit].babelpreamble(
+						*lit,
+						buffer().params().encoding(),
+						use_polyglossia));
 		}
 	}
 	if ((use_babel || use_polyglossia) && !UsedLanguages_.empty()) {
@@ -1206,14 +1225,11 @@ docstring const LaTeXFeatures::getTClassI18nPreamble(bool use_babel, bool use_po
 			docstring name = buffer().language()->translateLayout(fl.name());
 			// only request translation if we have a real translation
 			// (that differs from the source)
-			if (use_polyglossia && flname != name)
+			if (flname != name)
 				snippets.insert(getFloatI18nPreamble(
-					type, name,
-					from_ascii(buffer().language()->polyglossia())));
-			else if (flname != name)
-				snippets.insert(getFloatI18nPreamble(
-					type, name,
-					from_ascii(buffer().language()->babel())));
+						type, name, buffer().language(),
+						buffer().params().encoding(),
+						use_polyglossia));
 			for (lang_it lit = lbeg; lit != lend; ++lit) {
 				string const code = (*lit)->code();
 				name = (*lit)->translateLayout(fl.name());
@@ -1224,14 +1240,11 @@ docstring const LaTeXFeatures::getTClassI18nPreamble(bool use_babel, bool use_po
 				// something different to the English source.
 				bool const have_translation =
 					(flname != name || contains(code, "en"));
-				if (use_polyglossia && have_translation)
+				if (have_translation)
 					snippets.insert(getFloatI18nPreamble(
-						type, name,
-						from_ascii((*lit)->polyglossia())));
-				else if (have_translation)
-					snippets.insert(getFloatI18nPreamble(
-						type, name,
-						from_ascii((*lit)->babel())));
+						type, name, *lit,
+						buffer().params().encoding(),
+						use_polyglossia));
 			}
 		}
 	}
