@@ -246,8 +246,10 @@ static bool doInsertInset(Cursor & cur, Text * text,
 			inset->edit(cur, true);
 		// Now put this into inset
 		Font const f(inherit_font, cur.current_font.language());
-		cur.text()->insertStringAsLines(cur, ds, f);
-		cur.leaveInset(*inset);
+		if (!ds.empty()) {
+			cur.text()->insertStringAsLines(cur, ds, f);
+			cur.leaveInset(*inset);
+		}
 		return true;
 	}
 
@@ -1893,7 +1895,7 @@ void Text::dispatch(Cursor & cur, FuncRequest & cmd)
 			break;
 		selectWordWhenUnderCursor(cur, WHOLE_WORD_STRICT);
 		Font font(ignore_font, lang);
-		toggleAndShow(cur, this, font);
+		toggleAndShow(cur, this, font, false);
 		break;
 	}
 
