@@ -723,12 +723,6 @@ bool MenuDefinition::searchMenu(FuncRequest const & func, docstring_list & names
 }
 
 
-bool compareFormat(Format const * p1, Format const * p2)
-{
-	return *p1 < *p2;
-}
-
-
 QString limitStringLength(docstring const & str)
 {
 	size_t const max_item_length = 45;
@@ -1018,7 +1012,6 @@ void MenuDefinition::expandFormats(MenuItem::Kind kind, Buffer const * buf)
 		formats = buf->params().exportableFormats(false);
 		action = LFUN_BUFFER_EXPORT;
 	}
-	sort(formats.begin(), formats.end(), &compareFormat);
 
 	bool const view_update = (kind == MenuItem::ViewFormats
 			|| kind == MenuItem::UpdateFormats);
