@@ -465,7 +465,8 @@ public:
 	///
 	bool isMultiColumn(idx_type cell) const;
 	///
-	idx_type setMultiColumn(idx_type cell, idx_type number);
+	idx_type setMultiColumn(idx_type cell, idx_type number,
+			     bool const right_border);
 	///
 	void unsetMultiColumn(idx_type cell);
 	///
@@ -475,7 +476,8 @@ public:
 	///
 	bool isMultiRow(idx_type cell) const;
 	///
-	idx_type setMultiRow(idx_type cell, idx_type number);
+	idx_type setMultiRow(idx_type cell, idx_type number,
+			     bool const bottom_border);
 	///
 	void unsetMultiRow(idx_type cell);
 	///
