@@ -663,7 +663,7 @@ void InsetMathHull::label(row_type row, docstring const & label)
 			label_[row] = dummy_pointer;
 		} else {
 			if (buffer_)
-				label_[row]->updateCommand(label);
+				label_[row]->updateLabelAndRefs(label);
 			else
 				label_[row]->setParam("name", label);
 		}
@@ -1880,7 +1880,7 @@ bool InsetMathHull::readQuiet(Lexer & lex)
 }
 
 
-int InsetMathHull::plaintext(odocstream & os, OutputParams const &) const
+int InsetMathHull::plaintext(odocstream & os, OutputParams const & op) const
 {
 	// disables ASCII-art for export of equations. See #2275.
 	if (0 && display()) {
@@ -1893,24 +1893,29 @@ int InsetMathHull::plaintext(odocstream & os, OutputParams const &) const
 		// reset metrics cache to "real" values
 		//metrics();
 		return tpain.textheight();
-	} else {
-		odocstringstream oss;
-		Encoding const * const enc = encodings.fromLyXName("utf8");
-		WriteStream wi(oss, false, true, WriteStream::wsDefault, enc);
-		// Fix Bug #6139
-		if (type_ == hullRegexp)
-			write(wi);
-		else {
-			for (row_type r = 0; r < nrows(); ++r) {
-				for (col_type c = 0; c < ncols(); ++c)
-					wi << (c == 0 ? "" : "\t") << cell(index(r, c));
-				wi << "\n";
-			}
-		}
-		docstring const str = oss.str();
-		os << str;
-		return str.size();
 	}
+
+	odocstringstream oss;
+	Encoding const * const enc = encodings.fromLyXName("utf8");
+	WriteStream wi(oss, false, true, WriteStream::wsDefault, enc);
+
+	// Fix Bug #6139
+	if (type_ == hullRegexp)
+		write(wi);
+	else {
+		for (row_type r = 0; r < nrows(); ++r) {
+			for (col_type c = 0; c < ncols(); ++c)
+				wi << (c == 0 ? "" : "\t") << cell(index(r, c));
+			// if it's for the TOC, we write just the first line
+			// and do not include the newline.
+			if (op.for_toc)
+				break;
+			wi << "\n";
+		}
+	}
+	docstring const str = oss.str();
+	os << str;
+	return str.size();
 }
 
 
@@ -2226,7 +2231,9 @@ void InsetMathHull::toString(odocstream & os) const
 void InsetMathHull::forToc(docstring & os, size_t) const
 {
 	odocstringstream ods;
-	plaintext(ods, OutputParams(0));
+	OutputParams op(0);
+	op.for_toc = true;
+	plaintext(ods, op);
 	os += ods.str();
 }
 
