@@ -1390,6 +1390,14 @@ bool InsetMathNest::getStatus(Cursor & cur, FuncRequest const & cmd,
 		break;
 	}
 
+	case LFUN_DIALOG_SHOW_NEW_INSET: {
+		docstring const & name = cmd.argument();
+		if (name == "space")
+			flag.setEnabled(false);
+		break;
+	}
+
+
 	case LFUN_MATH_DELIM:
 	case LFUN_MATH_BIGDELIM:
 		// Don't do this with multi-cell selections
@@ -1410,6 +1418,13 @@ bool InsetMathNest::getStatus(Cursor & cur, FuncRequest const & cmd,
 		flag.setEnabled(false);
 		break;
 
+	case LFUN_SPACE_INSERT: {
+		docstring const & name = cmd.argument();
+		if (name == "protected" || name == "normal")
+			flag.setEnabled(false);
+		break;
+	}
+
 	case LFUN_INSET_DISSOLVE:
 		flag.setEnabled(!asHullInset());
 		break;
