@@ -3,7 +3,7 @@
  * This file is part of LyX, the document processor.
  * Licence details can be found in the file COPYING.
  *
- * \author Andr Pnitz
+ * \author André Pönitz
  *
  * Full author contact details are available in file CREDITS.
  */
@@ -16,6 +16,7 @@
 
 #include "MathExtern.h"
 
+#include "InsetMathAMSArray.h"
 #include "InsetMathArray.h"
 #include "InsetMathChar.h"
 #include "InsetMathDelim.h"
@@ -50,6 +51,18 @@ using namespace lyx::support;
 
 namespace lyx {
 
+namespace {
+
+enum ExternalMath {
+	HTML,
+	MAPLE,
+	MAXIMA,
+	MATHEMATICA,
+	MATHML,
+	OCTAVE
+};
+
+
 static char const * function_names[] = {
 	"arccos", "arcsin", "arctan", "arg", "bmod",
 	"cos", "cosh", "cot", "coth", "csc", "deg",
@@ -96,7 +109,8 @@ bool extractScript(MathData & ar,
 // try to extract an "argument" to some function.
 // returns position behind the argument
 MathData::iterator extractArgument(MathData & ar,
-	MathData::iterator pos, MathData::iterator last, bool function = false)
+	MathData::iterator pos, MathData::iterator last, 
+	ExternalMath kind, bool function = false)
 {
 	// nothing to get here
 	if (pos == last)
@@ -105,7 +119,9 @@ MathData::iterator extractArgument(MathData & ar,
 	// something delimited _is_ an argument
 	if ((*pos)->asDelimInset()) {
 		// leave out delimiters if this is a function argument
-		if (function) {
+		// unless we are doing MathML, in which case we do want
+		// the delimiters
+		if (function && kind != MATHML && kind != HTML) {
 			MathData const & arg = (*pos)->asDelimInset()->cell(0);
 			MathData::const_iterator cur = arg.begin();
 			MathData::const_iterator end = arg.end();
@@ -181,20 +197,31 @@ void extractMatrices(MathData & ar)
 	//lyxerr << "\nMatrices from: " << ar << endl;
 	// first pass for explicitly delimited stuff
 	for (size_t i = 0; i < ar.size(); ++i) {
-		if (!ar[i]->asDelimInset())
+		InsetMathDelim const * const inset = ar[i]->asDelimInset();
+		if (!inset)
 			continue;
-		MathData const & arr = ar[i]->asDelimInset()->cell(0);
+		MathData const & arr = inset->cell(0);
 		if (arr.size() != 1)
 			continue;
 		if (!arr.front()->asGridInset())
 			continue;
-		ar[i] = MathAtom(new InsetMathMatrix(*(arr.front()->asGridInset())));
+		ar[i] = MathAtom(new InsetMathMatrix(*(arr.front()->asGridInset()), 
+		                 inset->left_, inset->right_));
 	}
 
 	// second pass for AMS "pmatrix" etc
-	for (size_t i = 0; i < ar.size(); ++i)
-		if (ar[i]->asAMSArrayInset())
-			ar[i] = MathAtom(new InsetMathMatrix(*(ar[i]->asGridInset())));
+	for (size_t i = 0; i < ar.size(); ++i) {
+		InsetMathAMSArray const * const inset = ar[i]->asAMSArrayInset();
+		if (inset) {
+			string left = inset->name_left();
+			if (left == "Vert")
+				left = "[";
+			string right = inset->name_right();
+			if (right == "Vert")
+				right = "]";
+			ar[i] = MathAtom(new InsetMathMatrix(*inset, from_ascii(left), from_ascii(right)));
+		}
+	}
 	//lyxerr << "\nMatrices to: " << ar << endl;
 }
 
@@ -247,23 +274,6 @@ bool extractFunctionName(MathAtom const & at, docstring & str)
 }
 
 
-// convert this inset somehow to a number
-bool extractNumber(MathData const & ar, int & i)
-{
-	idocstringstream is(charSequence(ar.begin(), ar.end()));
-	is >> i;
-	return is;
-}
-
-
-bool extractNumber(MathData const & ar, double & d)
-{
-	idocstringstream is(charSequence(ar.begin(), ar.end()));
-	is >> d;
-	return is;
-}
-
-
 bool testString(MathAtom const & at, docstring const & str)
 {
 	docstring s;
@@ -301,9 +311,9 @@ void replaceNested(
 	MathData & ar,
 	TestItemFunc testOpen,
 	TestItemFunc testClose,
-	ReplaceArgumentFunc replaceArg
-)
+	ReplaceArgumentFunc replaceArg)
 {
+	Buffer * buf = ar.buffer();
 	// use indices rather than iterators for the loop  because we are going
 	// to modify the array.
 	for (size_t i = 0; i < ar.size(); ++i) {
@@ -318,7 +328,7 @@ void replaceNested(
 			continue;
 
 		// replace the original stuff by the new inset
-		ar[i] = replaceArg(MathData(it + 1, jt));
+		ar[i] = replaceArg(MathData(buf, it + 1, jt));
 		ar.erase(it + 1, jt + 1);
 	}
 }
@@ -332,6 +342,7 @@ void replaceNested(
 
 void splitScripts(MathData & ar)
 {
+	Buffer * buf = ar.buffer();
 	//lyxerr << "\nScripts from: " << ar << endl;
 	for (size_t i = 0; i < ar.size(); ++i) {
 		InsetMathScript const * script = ar[i]->asScriptInset();
@@ -354,7 +365,7 @@ void splitScripts(MathData & ar)
 
 		// create extra script inset and move superscript over
 		InsetMathScript * p = ar[i].nucleus()->asScriptInset();
-		auto_ptr<InsetMathScript> q(new InsetMathScript(true));
+		auto_ptr<InsetMathScript> q(new InsetMathScript(buf, true));
 		swap(q->up(), p->up());
 		p->removeScript(true);
 
@@ -382,6 +393,7 @@ void splitScripts(MathData & ar)
 
 void extractExps(MathData & ar)
 {
+	Buffer * buf = ar.buffer();
 	//lyxerr << "\nExps from: " << ar << endl;
 	for (size_t i = 0; i + 1 < ar.size(); ++i) {
 		// is this 'e'?
@@ -394,7 +406,7 @@ void extractExps(MathData & ar)
 			continue;
 
 		// create a proper exp-inset as replacement
-		ar[i] = MathAtom(new InsetMathExFunc(from_ascii("exp"), sup->cell(1)));
+		ar[i] = MathAtom(new InsetMathExFunc(buf, from_ascii("exp"), sup->cell(1)));
 		ar.erase(i + 1);
 	}
 	//lyxerr << "\nExps to: " << ar << endl;
@@ -406,6 +418,7 @@ void extractExps(MathData & ar)
 //
 void extractDets(MathData & ar)
 {
+	Buffer * buf = ar.buffer();
 	//lyxerr << "\ndet from: " << ar << endl;
 	for (MathData::iterator it = ar.begin(); it != ar.end(); ++it) {
 		InsetMathDelim const * del = (*it)->asDelimInset();
@@ -413,7 +426,7 @@ void extractDets(MathData & ar)
 			continue;
 		if (!del->isAbs())
 			continue;
-		*it = MathAtom(new InsetMathExFunc(from_ascii("det"), del->cell(0)));
+		*it = MathAtom(new InsetMathExFunc(buf, from_ascii("det"), del->cell(0)));
 	}
 	//lyxerr << "\ndet to: " << ar << endl;
 }
@@ -480,7 +493,8 @@ bool testCloseParen(MathAtom const & at)
 
 MathAtom replaceParenDelims(const MathData & ar)
 {
-	return MathAtom(new InsetMathDelim(from_ascii("("), from_ascii(")"), ar));
+	return MathAtom(new InsetMathDelim(const_cast<Buffer *>(ar.buffer()),
+		from_ascii("("), from_ascii(")"), ar));
 }
 
 
@@ -498,7 +512,8 @@ bool testCloseBracket(MathAtom const & at)
 
 MathAtom replaceBracketDelims(const MathData & ar)
 {
-	return MathAtom(new InsetMathDelim(from_ascii("["), from_ascii("]"), ar));
+	return MathAtom(new InsetMathDelim(const_cast<Buffer *>(ar.buffer()),
+		from_ascii("["), from_ascii("]"), ar));
 }
 
 
@@ -520,12 +535,14 @@ void extractDelims(MathData & ar)
 
 // replace 'f' '(...)' and 'f' '^n' '(...)' sequences by a real InsetMathExFunc
 // assume 'extractDelims' ran before
-void extractFunctions(MathData & ar)
+void extractFunctions(MathData & ar, ExternalMath kind)
 {
 	// we need at least two items...
 	if (ar.size() < 2)
 		return;
 
+	Buffer * buf = ar.buffer();
+
 	//lyxerr << "\nFunctions from: " << ar << endl;
 	for (size_t i = 0; i + 1 < ar.size(); ++i) {
 		MathData::iterator it = ar.begin() + i;
@@ -556,10 +573,11 @@ void extractFunctions(MathData & ar)
 		extractScript(exp, jt, ar.end(), true);
 
 		// create a proper inset as replacement
-		auto_ptr<InsetMathExFunc> p(new InsetMathExFunc(name));
+		auto_ptr<InsetMathExFunc> p(new InsetMathExFunc(buf, name));
 
 		// jt points to the "argument". Get hold of this.
-		MathData::iterator st = extractArgument(p->cell(0), jt, ar.end(), true);
+		MathData::iterator st = 
+				extractArgument(p->cell(0), jt, ar.end(), kind, true);
 
 		// replace the function name by a real function inset
 		*it = MathAtom(p.release());
@@ -615,12 +633,14 @@ bool testIntDiff(MathAtom const & at)
 
 // replace '\int' ['_^'] x 'd''x'(...)' sequences by a real InsetMathExInt
 // assume 'extractDelims' ran before
-void extractIntegrals(MathData & ar)
+void extractIntegrals(MathData & ar, ExternalMath kind)
 {
 	// we need at least three items...
 	if (ar.size() < 3)
 		return;
 
+	Buffer * buf = ar.buffer();
+
 	//lyxerr << "\nIntegrals from: " << ar << endl;
 	for (size_t i = 0; i + 1 < ar.size(); ++i) {
 		MathData::iterator it = ar.begin() + i;
@@ -638,17 +658,17 @@ void extractIntegrals(MathData & ar)
 			continue;
 
 		// core ist part from behind the scripts to the 'd'
-		auto_ptr<InsetMathExInt> p(new InsetMathExInt(from_ascii("int")));
+		auto_ptr<InsetMathExInt> p(new InsetMathExInt(buf, from_ascii("int")));
 
 		// handle scripts if available
 		if (!testIntSymbol(*it)) {
 			p->cell(2) = (*it)->asScriptInset()->down();
 			p->cell(3) = (*it)->asScriptInset()->up();
 		}
-		p->cell(0) = MathData(it + 1, jt);
+		p->cell(0) = MathData(buf, it + 1, jt);
 
 		// use the "thing" behind the 'd' as differential
-		MathData::iterator tt = extractArgument(p->cell(1), jt + 1, ar.end());
+		MathData::iterator tt = extractArgument(p->cell(1), jt + 1, ar.end(), kind);
 
 		// remove used parts
 		ar.erase(it + 1, tt);
@@ -712,6 +732,8 @@ void extractSums(MathData & ar)
 	if (ar.size() < 2)
 		return;
 
+	Buffer * buf = ar.buffer();
+
 	//lyxerr << "\nSums from: " << ar << endl;
 	for (size_t i = 0; i + 1 < ar.size(); ++i) {
 		MathData::iterator it = ar.begin() + i;
@@ -721,7 +743,7 @@ void extractSums(MathData & ar)
 			continue;
 
 		// create a proper inset as replacement
-		auto_ptr<InsetMathExInt> p(new InsetMathExInt(from_ascii("sum")));
+		auto_ptr<InsetMathExInt> p(new InsetMathExInt(buf, from_ascii("sum")));
 
 		// collect lower bound and summation index
 		InsetMathScript const * sub = ar[i]->asScriptInset();
@@ -733,8 +755,8 @@ void extractSums(MathData & ar)
 			if (xt != ar.end()) {
 				// we found a '=', use everything in front of that as index,
 				// and everything behind as lower index
-				p->cell(1) = MathData(ar.begin(), xt);
-				p->cell(2) = MathData(xt + 1, ar.end());
+				p->cell(1) = MathData(buf, ar.begin(), xt);
+				p->cell(2) = MathData(buf, xt + 1, ar.end());
 			} else {
 				// use everything as summation index, don't use scripts.
 				p->cell(1) = ar;
@@ -793,6 +815,7 @@ bool testDiffFrac(MathAtom const & at)
 
 void extractDiff(MathData & ar)
 {
+	Buffer * buf = ar.buffer();
 	//lyxerr << "\nDiffs from: " << ar << endl;
 	for (size_t i = 0; i < ar.size(); ++i) {
 		MathData::iterator it = ar.begin() + i;
@@ -808,7 +831,7 @@ void extractDiff(MathData & ar)
 		}
 
 		// create a proper diff inset
-		auto_ptr<InsetMathDiff> diff(new InsetMathDiff);
+		auto_ptr<InsetMathDiff> diff(new InsetMathDiff(buf));
 
 		// collect function, let jt point behind last used item
 		MathData::iterator jt = it + 1;
@@ -820,13 +843,13 @@ void extractDiff(MathData & ar)
 			// FIXME
 			//n = 1;
 			if (numer.size() > 2)
-				diff->cell(0) = MathData(numer.begin() + 2, numer.end());
+				diff->cell(0) = MathData(buf, numer.begin() + 2, numer.end());
 			else
 				jt = extractTerm(diff->cell(0), jt, ar.end());
 		} else {
 			// simply d f(x) / d... or  d/d...
 			if (numer.size() > 1)
-				diff->cell(0) = MathData(numer.begin() + 1, numer.end());
+				diff->cell(0) = MathData(buf, numer.begin() + 1, numer.end());
 			else
 				jt = extractTerm(diff->cell(0), jt, ar.end());
 		}
@@ -848,11 +871,11 @@ void extractDiff(MathData & ar)
 				if (extractNumber(script->up(), mult)) {
 					//lyxerr << "mult: " << mult << endl;
 					for (int i = 0; i < mult; ++i)
-						diff->addDer(MathData(dt + 1, st));
+						diff->addDer(MathData(buf, dt + 1, st));
 				}
 			} else {
 				// just  d.../dx
-				diff->addDer(MathData(dt + 1, et));
+				diff->addDer(MathData(buf, dt + 1, et));
 			}
 			dt = et;
 		}
@@ -881,6 +904,7 @@ bool testRightArrow(MathAtom const & at)
 // assume 'extractDelims' ran before
 void extractLims(MathData & ar)
 {
+	Buffer * buf = ar.buffer();
 	//lyxerr << "\nLimits from: " << ar << endl;
 	for (size_t i = 0; i < ar.size(); ++i) {
 		MathData::iterator it = ar.begin() + i;
@@ -901,8 +925,8 @@ void extractLims(MathData & ar)
 			continue;
 
 		// the -> splits the subscript int x and x0
-		MathData x  = MathData(s.begin(), st);
-		MathData x0 = MathData(st + 1, s.end());
+		MathData x  = MathData(buf, s.begin(), st);
+		MathData x0 = MathData(buf, st + 1, s.end());
 
 		// use something behind the script as core
 		MathData f;
@@ -912,7 +936,7 @@ void extractLims(MathData & ar)
 		ar.erase(it + 1, tt);
 
 		// create a proper inset as replacement
-		*it = MathAtom(new InsetMathLim(f, x, x0));
+		*it = MathAtom(new InsetMathLim(buf, f, x, x0));
 	}
 	//lyxerr << "\nLimits to: " << ar << endl;
 }
@@ -922,99 +946,29 @@ void extractLims(MathData & ar)
 // combine searches
 //
 
-void extractStructure(MathData & ar)
+void extractStructure(MathData & ar, ExternalMath kind)
 {
 	//lyxerr << "\nStructure from: " << ar << endl;
-	splitScripts(ar);
+	if (kind != MATHML && kind != HTML)
+		splitScripts(ar);
 	extractDelims(ar);
-	extractIntegrals(ar);
-	extractSums(ar);
+	extractIntegrals(ar, kind);
+	if (kind != MATHML && kind != HTML)
+		extractSums(ar);
 	extractNumbers(ar);
 	extractMatrices(ar);
-	extractFunctions(ar);
-	extractDets(ar);
-	extractDiff(ar);
-	extractExps(ar);
-	extractLims(ar);
-	extractStrings(ar);
+	extractFunctions(ar, kind);
+	if (kind != MATHML && kind != HTML) {
+		extractDets(ar);
+		extractDiff(ar);
+		extractExps(ar);
+		extractLims(ar);
+		extractStrings(ar);
+	}
 	//lyxerr << "\nStructure to: " << ar << endl;
 }
 
 
-void write(MathData const & dat, WriteStream & wi)
-{
-	MathData ar = dat;
-	extractStrings(ar);
-	wi.firstitem() = true;
-	for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it) {
-		(*it)->write(wi);
-		wi.firstitem() = false;
-	}
-}
-
-
-void normalize(MathData const & ar, NormalStream & os)
-{
-	for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
-		(*it)->normalize(os);
-}
-
-
-void octave(MathData const & dat, OctaveStream & os)
-{
-	MathData ar = dat;
-	extractStructure(ar);
-	for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
-		(*it)->octave(os);
-}
-
-
-void maple(MathData const & dat, MapleStream & os)
-{
-	MathData ar = dat;
-	extractStructure(ar);
-	for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
-		(*it)->maple(os);
-}
-
-
-void maxima(MathData const & dat, MaximaStream & os)
-{
-	MathData ar = dat;
-	extractStructure(ar);
-	for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
-		(*it)->maxima(os);
-}
-
-
-void mathematica(MathData const & dat, MathematicaStream & os)
-{
-	MathData ar = dat;
-	extractStructure(ar);
-	for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
-		(*it)->mathematica(os);
-}
-
-
-void mathmlize(MathData const & dat, MathStream & os)
-{
-	MathData ar = dat;
-	extractStructure(ar);
-	if (ar.size() == 0)
-		os << "<mrow/>";
-	else if (ar.size() == 1)
-		os << ar.front();
-	else {
-		os << MTag("mrow");
-		for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
-			(*it)->mathmlize(os);
-		os << ETag("mrow");
-	}
-}
-
-
-
-
 namespace {
 
 	string captureOutput(string const & cmd, string const & data)
@@ -1254,7 +1208,8 @@ namespace {
 		vs << ar;
 		string expr = to_utf8(os.str());
 		string out;
-
+		// FIXME const cast
+		Buffer * buf = const_cast<Buffer *>(ar.buffer());
 		lyxerr << "pipe: ar: '" << ar << "'\n"
 		       << "pipe: expr: '" << expr << "'" << endl;
 
@@ -1304,13 +1259,14 @@ namespace {
 		out = out.substr(i + 6);
 
 		// parse output as matrix or single number
-		MathAtom at(new InsetMathArray(from_ascii("array"), from_utf8(out)));
+		MathAtom at(new InsetMathArray(buf, from_ascii("array"), from_utf8(out)));
 		InsetMathArray const * mat = at->asArrayInset();
-		MathData res;
+		MathData res(buf);
 		if (mat->ncols() == 1 && mat->nrows() == 1)
 			res.append(mat->cell(0));
 		else {
-			res.push_back(MathAtom(new InsetMathDelim(from_ascii("("), from_ascii(")"))));
+			res.push_back(MathAtom(
+				new InsetMathDelim(buf, from_ascii("("), from_ascii(")"))));
 			res.back().nucleus()->cell(0).push_back(at);
 		}
 		return res;
@@ -1402,6 +1358,111 @@ namespace {
 
 }
 
+} // anon namespace
+
+void write(MathData const & dat, WriteStream & wi)
+{
+	MathData ar = dat;
+	extractStrings(ar);
+	wi.firstitem() = true;
+	for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it) {
+		(*it)->write(wi);
+		wi.firstitem() = false;
+	}
+}
+
+
+void normalize(MathData const & ar, NormalStream & os)
+{
+	for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
+		(*it)->normalize(os);
+}
+
+
+void octave(MathData const & dat, OctaveStream & os)
+{
+	MathData ar = dat;
+	extractStructure(ar, OCTAVE);
+	for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
+		(*it)->octave(os);
+}
+
+
+void maple(MathData const & dat, MapleStream & os)
+{
+	MathData ar = dat;
+	extractStructure(ar, MAPLE);
+	for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
+		(*it)->maple(os);
+}
+
+
+void maxima(MathData const & dat, MaximaStream & os)
+{
+	MathData ar = dat;
+	extractStructure(ar, MAXIMA);
+	for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
+		(*it)->maxima(os);
+}
+
+
+void mathematica(MathData const & dat, MathematicaStream & os)
+{
+	MathData ar = dat;
+	extractStructure(ar, MATHEMATICA);
+	for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
+		(*it)->mathematica(os);
+}
+
+
+void mathmlize(MathData const & dat, MathStream & os)
+{
+	MathData ar = dat;
+	extractStructure(ar, MATHML);
+	if (ar.size() == 0)
+		os << "<mrow/>";
+	else if (ar.size() == 1)
+		os << ar.front();
+	else {
+		os << MTag("mrow");
+		for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
+			(*it)->mathmlize(os);
+		os << ETag("mrow");
+	}
+}
+
+
+void htmlize(MathData const & dat, HtmlStream & os)
+{
+	MathData ar = dat;
+	extractStructure(ar, HTML);
+	if (ar.size() == 0) 
+		return;
+	if (ar.size() == 1) {
+		os << ar.front();
+		return;
+	}
+	for (MathData::const_iterator it = ar.begin(); it != ar.end(); ++it)
+		(*it)->htmlize(os);
+}
+
+
+// convert this inset somehow to a number
+bool extractNumber(MathData const & ar, int & i)
+{
+	idocstringstream is(charSequence(ar.begin(), ar.end()));
+	is >> i;
+	return is;
+}
+
+
+bool extractNumber(MathData const & ar, double & d)
+{
+	idocstringstream is(charSequence(ar.begin(), ar.end()));
+	is >> d;
+	return is;
+}
+
 
 MathData pipeThroughExtern(string const & lang, docstring const & extra,
 	MathData const & ar)
@@ -1435,7 +1496,7 @@ MathData pipeThroughExtern(string const & lang, docstring const & extra,
 	}
 
 	// run external sript
-	string out = captureOutput(file.absFilename(), data);
+	string out = captureOutput(file.absFileName(), data);
 	MathData res;
 	mathed_parse_cell(res, from_utf8(out));
 	return res;
