@@ -14,7 +14,6 @@
 #include <config.h>
 
 #include "support/debug.h"
-#include "support/environment.h"
 #include "support/filetools.h"
 #include "support/lstrings.h"
 #include "support/qstring_helpers.h"
@@ -101,24 +100,8 @@ ProgressInterface* ProgressInterface::instance()
 int Systemcall::startscript(Starttype how, string const & what,
 			    std::string const & path, bool /*process_events*/)
 {
-	string command;
-	string const texinputs = os::latex_path_list(
-			replaceCurdirPath(path, lyxrc.texinputs_prefix));
-	string const sep = string(1, os::path_separator(os::TEXENGINE));
-	string const env = getEnv("TEXINPUTS");
-
-	switch (os::shell()) {
-	case os::UNIX:
-		command = path.empty() || lyxrc.texinputs_prefix.empty() ? what
-			: "env TEXINPUTS='." + sep + texinputs
-					     + sep + env + "' " + what;
-		break;
-	case os::CMD_EXE:
-		command = path.empty() || lyxrc.texinputs_prefix.empty() ? what
-			: "set TEXINPUTS=." + sep + texinputs
-					    + sep + env + " & " + what;
-		break;
-	}
+	string command =
+		to_filesystem8bit(from_utf8(latexEnvCmdPrefix(path))) + what;
 
 	if (how == DontWait) {
 		switch (os::shell()) {
@@ -129,7 +112,8 @@ int Systemcall::startscript(Starttype how, string const & what,
 			command = "start /min " + command;
 			break;
 		}
-	}
+	} else if (os::shell() == os::CMD_EXE)
+		command = subst(command, "cmd /d /c ", "");
 
 	return ::system(command.c_str());
 }
@@ -228,7 +212,7 @@ int Systemcall::startscript(Starttype how, string const & what,
 			    string const & path, bool process_events)
 {
 	string outfile;
-	QString cmd = toqstr(parsecmd(what, outfile));
+	QString cmd = QString::fromLocal8Bit(parsecmd(what, outfile).c_str());
 
 	SystemcallPrivate d(outfile);
 
@@ -268,13 +252,12 @@ SystemcallPrivate::SystemcallPrivate(const std::string& of) :
                                 out_index_(0),
                                 err_index_(0),
                                 out_file_(of), 
-                                texinputs_(getEnv("TEXINPUTS")),
                                 process_events_(false)
 {
 	if (!out_file_.empty()) {
 		// Check whether we have to simply throw away the output.
 		if (out_file_ != os::nulldev())
-			process_->setStandardOutputFile(toqstr(out_file_));
+			process_->setStandardOutputFile(QString::fromLocal8Bit(out_file_.c_str()));
 	}
 
 	connect(process_, SIGNAL(readyReadStandardOutput()), SLOT(stdOut()));
@@ -285,27 +268,12 @@ SystemcallPrivate::SystemcallPrivate(const std::string& of) :
 }
 
 
-
 void SystemcallPrivate::startProcess(QString const & cmd, string const & path)
 {
 	cmd_ = cmd;
 	if (process_) {
-		if (!path.empty() && !lyxrc.texinputs_prefix.empty()) {
-			string const texinputs = os::latex_path_list(
-				replaceCurdirPath(path, lyxrc.texinputs_prefix));
-			string const sep = string(1,
-					os::path_separator(os::TEXENGINE));
-			string const prefix = "." + sep + texinputs + sep;
-			if (prefixIs(texinputs_, prefix))
-				texinputs_.clear();
-			else
-				setEnv("TEXINPUTS", prefix + texinputs_);
-		}
 		state = SystemcallPrivate::Starting;
-		if (os::shell() == os::CMD_EXE)
-			process_->start(QLatin1String("cmd /d /c ") + cmd_);
-		else
-			process_->start(cmd_);
+		process_->start(toqstr(latexEnvCmdPrefix(path)) + cmd_);
 	}
 }
 
@@ -362,9 +330,6 @@ bool SystemcallPrivate::waitWhile(State waitwhile, bool process_events, int time
 
 SystemcallPrivate::~SystemcallPrivate()
 {
-	if (!texinputs_.empty())
-		setEnv("TEXINPUTS", texinputs_);
-
 	if (out_index_) {
 		out_data_[out_index_] = '\0';
 		out_index_ = 0;
