@@ -49,19 +49,19 @@ const char * const modules_placeholder = "\001modules\001";
 // needed to handle encodings with babel
 bool one_language = true;
 string h_inputencoding = "auto";
+string h_paragraph_separation = "indent";
 
 namespace {
 
 //add this to known_languages when updating to lyxformat 266:
-// "armenian"
+// "armenian" (needs special handling since not supported by standard babel)
 //add these to known_languages when updating to lyxformat 268:
 //"chinese-simplified", "chinese-traditional", "japanese", "korean"
 // Both changes require first that support for non-babel languages (CJK,
 // armtex) is added.
-// add turkmen for lyxformat 383
 /**
  * known babel language names (including synonyms)
- * not in standard babel: arabic, arabtex, belarusian, serbian-latin, thai
+ * not in standard babel: arabic, arabtex, armenian, belarusian, serbian-latin, thai
  * not yet supported by LyX: kurmanji
  * please keep this in sync with known_coded_languages line by line!
  */
@@ -77,8 +77,9 @@ const char * const known_languages[] = {"acadian", "afrikaans", "albanian",
 "ngerman", "ngermanb", "norsk", "nynorsk", "polutonikogreek", "polish",
 "portuges", "portuguese", "romanian", "russian", "russianb", "samin",
 "scottish", "serbian", "serbian-latin", "slovak", "slovene", "spanish",
-"swedish", "thai", "turkish", "ukraineb", "ukrainian", "uppersorbian",
-"UKenglish", "USenglish", "usorbian", "vietnam", "welsh", 0};
+"swedish", "thai", "turkish", "turkmen", "ukraineb", "ukrainian",
+"uppersorbian", "UKenglish", "USenglish", "usorbian", "vietnam", "welsh",
+0};
 
 /**
  * the same as known_languages with .lyx names
@@ -96,21 +97,21 @@ const char * const known_coded_languages[] = {"french", "afrikaans", "albanian",
 "ngerman", "ngerman", "norsk", "nynorsk", "polutonikogreek", "polish",
 "portuguese", "portuguese", "romanian", "russian", "russian", "samin",
 "scottish", "serbian", "serbian-latin", "slovak", "slovene", "spanish",
-"swedish", "thai", "turkish", "ukrainian", "ukrainian", "uppersorbian",
-"uppersorbian", "english", "english", "vietnamese", "welsh", 0};
+"swedish", "thai", "turkish", "turkmen", "ukrainian", "ukrainian",
+"uppersorbian", "uppersorbian", "english", "english", "vietnamese", "welsh",
+0};
 
 /// languages with english quotes (.lyx names)
 const char * const known_english_quotes_languages[] = {"american", "bahasa",
 "bahasam", "brazilian", "canadian", "chinese-simplified", "english",
 "esperanto", "hebrew", "irish", "korean", "portuguese", "scottish", "thai", 0};
 
-//add this to known_french_quotes_languages when updating to
-//lyxformat 383: "turkmen"
 /// languages with french quotes (.lyx names)
 const char * const known_french_quotes_languages[] = {"albanian",
 "arabic_arabi", "arabic_arabtex", "basque", "canadien", "catalan", "french",
 "galician", "greek", "italian", "norsk", "nynorsk", "polutonikogreek",
-"russian", "spanish", "spanish-mexico", "turkish", "ukrainian", "vietnamese", 0};
+"russian", "spanish", "spanish-mexico", "turkish", "turkmen", "ukrainian",
+"vietnamese", 0};
 
 /// languages with german quotes (.lyx names)
 const char * const known_german_quotes_languages[] = {"austrian", "bulgarian",
@@ -125,10 +126,15 @@ const char * const known_polish_quotes_languages[] = {"afrikaans", "croatian",
 const char * const known_swedish_quotes_languages[] = {"finnish",
 "swedish", 0};
 
+/// known language packages from the times before babel
+const char * const known_old_language_packages[] = {"french", "frenchle",
+"frenchpro", "german", "ngerman", "pmfrench", 0};
+
 char const * const known_fontsizes[] = { "10pt", "11pt", "12pt", 0 };
 
-const char * const known_roman_fonts[] = { "ae", "bookman", "charter",
-"cmr", "fourier", "lmodern", "mathpazo", "mathptmx", "newcent", 0};
+const char * const known_roman_fonts[] = { "ae", "beraserif", "bookman",
+"ccfonts", "chancery", "charter", "cmr", "fourier", "lmodern", "mathpazo",
+"mathptmx", "newcent", "utopia", 0};
 
 const char * const known_sans_fonts[] = { "avant", "berasans", "cmbr", "cmss",
 "helvet", "lmss", 0};
@@ -137,9 +143,11 @@ const char * const known_typewriter_fonts[] = { "beramono", "cmtl", "cmtt",
 "courier", "lmtt", "luximono", "fourier", "lmodern", "mathpazo", "mathptmx",
 "newcent", 0};
 
-const char * const known_paper_sizes[] = { "a3paper", "b3paper", "a4paper",
-"b4paper", "a5paper", "b5paper", "executivepaper", "legalpaper",
-"letterpaper", 0};
+const char * const known_paper_sizes[] = { "a0paper", "b0paper", "c0paper",
+"a1paper", "b1paper", "c1paper", "a2paper", "b2paper", "c2paper", "a3paper",
+"b3paper", "c3paper", "a4paper", "b4paper", "c4paper", "a5paper", "b5paper",
+"c5paper", "a6paper", "b6paper", "c6paper", "executivepaper", "legalpaper",
+"letterpaper", "b0j", "b1j", "b2j", "b3j", "b4j", "b5j", "b6j", 0};
 
 const char * const known_class_paper_sizes[] = { "a4paper", "a5paper",
 "executivepaper", "legalpaper", "letterpaper", 0};
@@ -156,6 +164,12 @@ const char * const known_if_commands[] = {"if", "ifarydshln", "ifbraket",
 "ifcancel", "ifcolortbl", "ifeurosym", "ifmarginnote", "ifmmode", "ifpdf",
 "ifsidecap", "ifupgreek", 0};
 
+const char * const known_basic_colors[] = {"blue", "black", "cyan", "green",
+"magenta", "red", "white", "yellow", 0};
+
+const char * const known_basic_color_codes[] = {"#0000ff", "#000000", "#00ffff", "#00ff00",
+"#ff00ff", "#ff0000", "#ffffff", "#ffff00", 0};
+
 /// conditional commands with three arguments like \@ifundefined{}{}{}
 const char * const known_if_3arg_commands[] = {"@ifundefined", "IfFileExists",
 0};
@@ -166,6 +180,8 @@ string h_textclass               = "article";
 string h_use_default_options     = "false";
 string h_options;
 string h_language                = "english";
+string h_language_package        = "none";
+string h_fontencoding            = "default";
 string h_font_roman              = "default";
 string h_font_sans               = "default";
 string h_font_typewriter         = "default";
@@ -198,13 +214,19 @@ string h_papersize               = "default";
 string h_use_geometry            = "false";
 string h_use_amsmath             = "1";
 string h_use_esint               = "1";
+string h_use_mhchem              = "0";
+string h_use_mathdots            = "0";
 string h_cite_engine             = "basic";
 string h_use_bibtopic            = "false";
 string h_paperorientation        = "portrait";
+string h_suppress_date           = "false";
+string h_use_refstyle            = "0";
+string h_backgroundcolor;
+string h_boxbgcolor;
+string h_fontcolor;
 string h_notefontcolor;
 string h_secnumdepth             = "3";
 string h_tocdepth                = "3";
-string h_paragraph_separation    = "indent";
 string h_defskip                 = "medskip";
 string h_paragraph_indentation   = "default";
 string h_quotes_language         = "english";
@@ -214,6 +236,9 @@ string h_paperpagestyle          = "default";
 string h_listings_params;
 string h_tracking_changes        = "false";
 string h_output_changes          = "false";
+string h_html_math_output        = "0";
+string h_html_css_as_file        = "0";
+string h_html_be_strict          = "false";
 string h_margins;
 
 
@@ -483,10 +508,14 @@ void handle_package(Parser &p, string const & name, string const & opts,
 
 	// typewriter fonts
 	if (is_known(name, known_typewriter_fonts)) {
-		h_font_typewriter = name;
-		if (!opts.empty()) {
-			scale = opts;
-			h_font_tt_scale = scale_as_percentage(scale);
+		// fourier can be set as roman font _only_
+		// fourier as typewriter is handled in handling of \ttdefault
+		if (name != "fourier") {
+			h_font_typewriter = name;
+			if (!opts.empty()) {
+				scale = opts;
+				h_font_tt_scale = scale_as_percentage(scale);
+			}
 		}
 	}
 
@@ -494,36 +523,69 @@ void handle_package(Parser &p, string const & name, string const & opts,
 	if (name == "eco")
 		h_font_osf = "true";
 
+	// after the detection and handling of special cases, we can remove the
+	// fonts, otherwise they would appear in the preamble, see bug #7856
+	if (is_known(name, known_roman_fonts) || is_known(name, known_sans_fonts)
+		||	is_known(name, known_typewriter_fonts))
+		;
+
 	else if (name == "amsmath" || name == "amssymb")
 		h_use_amsmath = "2";
 
 	else if (name == "esint")
 		h_use_esint = "2";
 
-	else if (name == "babel" && !opts.empty()) {
-		// check if more than one option was used - used later for inputenc
-		// in case inputenc is parsed before babel, set the encoding to auto
-		if (options.begin() != options.end() - 1) {
-			one_language = false;
-			h_inputencoding = "auto";
+	else if (name == "mhchem")
+		h_use_mhchem = "2";
+
+	else if (name == "mathdots")
+		h_use_mathdots = "2";
+
+	else if (name == "babel") {
+		h_language_package = "default";
+		// One might think we would have to do nothing if babel is loaded
+		// without any options to prevent pollution of the preamble with this
+		// babel call in every roundtrip.
+		// But the user could have defined babel-specific things afterwards. So
+		// we need to keep it in the preamble to prevent cases like bug #7861.
+		if (!opts.empty()) {
+			// check if more than one option was used - used later for inputenc
+			if (options.begin() != options.end() - 1)
+				one_language = false;
+			// babel takes the last language of the option of its \usepackage
+			// call as document language. If there is no such language option, the
+			// last language in the documentclass options is used.
+			handle_opt(options, known_languages, h_language);
+			// If babel is called with options, LyX puts them by default into the
+			// document class options. This works for most languages, except
+			// for Latvian, Lithuanian, Mongolian, Turkmen and Vietnamese and
+			// perhaps in future others.
+			// Therefore keep the babel call as it is as the user might have
+			// reasons for it.
+			h_preamble << "\\usepackage[" << opts << "]{babel}\n";
+			delete_opt(options, known_languages);
 		}
-		// babel takes the last language of the option of its \usepackage
-		// call as document language. If there is no such language option, the
-		// last language in the documentclass options is used.
-		handle_opt(options, known_languages, h_language);
-		delete_opt(options, known_languages);
+		else
+			h_preamble << "\\usepackage{babel}\n";						
 	}
 
-	else if (name == "fontenc")
-		 ;// ignore this
+	else if (name == "fontenc") {
+		h_fontencoding = getStringFromVector(options, ",");
+		/* We could do the following for better round trip support,
+		 * but this makes the document less portable, so I skip it:
+		if (h_fontencoding == lyxrc.fontenc)
+			h_fontencoding = "global";
+		 */
+		options.clear();
+	}
 
 	else if (name == "inputenc" || name == "luainputenc") {
 		// h_inputencoding is only set when there is not more than one
 		// inputenc option because otherwise h_inputencoding must be
 		// set to "auto" (the default encoding of the document language)
 		// Therefore check for the "," character.
-		// It is also only set when there is not more then one babel
-		// language option but this is handled in the routine for babel.
+		// It is also only set when there is not more than one babel
+		// language option.
 		if (opts.find(",") == string::npos && one_language == true)
 			h_inputencoding = opts;
 		if (!options.empty())
@@ -531,6 +593,13 @@ void handle_package(Parser &p, string const & name, string const & opts,
 		options.clear();
 	}
 
+	else if (is_known(name, known_old_language_packages)) {
+		// known language packages from the times before babel
+		// if they are found and not also babel, they will be used as
+		// cutom language package
+		h_language_package = "\\usepackage{" + name + "}";
+	}
+
 	else if (name == "makeidx")
 		; // ignore this
 
@@ -552,7 +621,7 @@ void handle_package(Parser &p, string const & name, string const & opts,
 	else if (name == "url")
 		; // ignore this
 
-	else if (LYX_FORMAT >= 408 && name == "subscript")
+	else if (name == "subscript")
 		; // ignore this
 
 	else if (name == "color") {
@@ -567,10 +636,25 @@ void handle_package(Parser &p, string const & name, string const & opts,
 	else if (name == "setspace")
 		; // ignore this
 
+#if 0
+	// do not ignore as long as we don't support all commands (e.g. \xout is missing)
+	else if (name == "ulem")
+		; // ignore this
+#endif
+
 	else if (name == "geometry")
 		; // Ignore this, the geometry settings are made by the \geometry
 		  // command. This command is handled below.
 
+	else if (name == "rotfloat")
+		; // ignore this
+
+	else if (name == "wrapfig")
+		; // ignore this
+
+	else if (name == "subfig")
+		; // ignore this
+
 	else if (is_known(name, known_languages))
 		h_language = name;
 
@@ -677,7 +761,9 @@ void end_preamble(ostream & os, TextClass const & /*textclass*/)
 	os << "\\use_default_options " << h_use_default_options << "\n"
 	   << modules_placeholder
 	   << "\\language " << h_language << "\n"
+	   << "\\language_package " << h_language_package << "\n"
 	   << "\\inputencoding " << h_inputencoding << "\n"
+	   << "\\fontencoding " << h_fontencoding << "\n"
 	   << "\\font_roman " << h_font_roman << "\n"
 	   << "\\font_sans " << h_font_sans << "\n"
 	   << "\\font_typewriter " << h_font_typewriter << "\n"
@@ -719,16 +805,26 @@ void end_preamble(ostream & os, TextClass const & /*textclass*/)
 	   << "\\use_geometry " << h_use_geometry << "\n"
 	   << "\\use_amsmath " << h_use_amsmath << "\n"
 	   << "\\use_esint " << h_use_esint << "\n"
+	   << "\\use_mhchem " << h_use_mhchem << "\n"
+	   << "\\use_mathdots " << h_use_mathdots << "\n"
 	   << "\\cite_engine " << h_cite_engine << "\n"
 	   << "\\use_bibtopic " << h_use_bibtopic << "\n"
-	   << "\\paperorientation " << h_paperorientation << '\n';
-	if (LYX_FORMAT >= 382 && !h_notefontcolor.empty())
+	   << "\\paperorientation " << h_paperorientation << '\n'
+	   << "\\suppress_date " << h_suppress_date << '\n'
+	   << "\\use_refstyle " << h_use_refstyle << '\n';
+	if (!h_fontcolor.empty())
+		os << "\\fontcolor " << h_fontcolor << '\n';
+	if (!h_notefontcolor.empty())
 		os << "\\notefontcolor " << h_notefontcolor << '\n';
+	if (!h_backgroundcolor.empty())
+		os << "\\backgroundcolor " << h_backgroundcolor << '\n';
+	if (!h_boxbgcolor.empty())
+		os << "\\boxbgcolor " << h_boxbgcolor << '\n';
 	os << h_margins
 	   << "\\secnumdepth " << h_secnumdepth << "\n"
 	   << "\\tocdepth " << h_tocdepth << "\n"
 	   << "\\paragraph_separation " << h_paragraph_separation << "\n";
-	if (LYX_FORMAT < 365 || h_paragraph_separation == "skip")
+	if (h_paragraph_separation == "skip")
 		os << "\\defskip " << h_defskip << "\n";
 	else
 		os << "\\paragraph_indentation " << h_paragraph_indentation << "\n";
@@ -740,6 +836,9 @@ void end_preamble(ostream & os, TextClass const & /*textclass*/)
 		os << "\\listings_params " << h_listings_params << "\n";
 	os << "\\tracking_changes " << h_tracking_changes << "\n"
 	   << "\\output_changes " << h_output_changes << "\n"
+	   << "\\html_math_output " << h_html_math_output << "\n"
+	   << "\\html_css_as_file " << h_html_css_as_file << "\n"
+	   << "\\html_be_strict " << h_html_be_strict << "\n"
 	   << "\\end_header\n\n"
 	   << "\\begin_body\n";
 	// clear preamble for subdocuments
@@ -825,6 +924,46 @@ void parse_preamble(Parser & p, ostream & os,
 		else if (t.cs() == "pagestyle")
 			h_paperpagestyle = p.verbatim_item();
 
+		else if (t.cs() == "date") {
+			string argument = p.getArg('{', '}');
+			if (argument.empty())
+				h_suppress_date = "true";
+			else
+				h_preamble << t.asInput() << '{' << argument << '}';
+		}
+
+		else if (t.cs() == "color") {
+			string argument = p.getArg('{', '}');
+			// check the case that a standard color is used
+			if (is_known(argument, known_basic_colors))
+				h_fontcolor = color2code(argument);
+			// check the case that LyX's document_fontcolor is defined
+			// but not used for \color
+			if (argument != "document_fontcolor"
+				&& !is_known(argument, known_basic_colors)) {
+				h_preamble << t.asInput() << '{' << argument << '}';
+				// the color might already be set because \definecolor
+				// is parsed before this
+				h_fontcolor = "";
+			}
+		}
+
+		else if (t.cs() == "pagecolor") {
+			string argument = p.getArg('{', '}');
+			// check the case that a standard color is used
+			if (is_known(argument, known_basic_colors))
+				h_backgroundcolor = color2code(argument);
+			// check the case that LyX's page_backgroundcolor is defined
+			// but not used for \pagecolor
+			if (argument != "page_backgroundcolor"
+				&& !is_known(argument, known_basic_colors)) {
+				h_preamble << t.asInput() << '{' << argument << '}';
+				// the color might already be set because \definecolor
+				// is parsed before this
+				h_backgroundcolor = "";
+			}
+		}
+
 		else if (t.cs() == "makeatletter") {
 			// LyX takes care of this
 			p.setCatCode('@', catLetter);
@@ -1018,11 +1157,8 @@ void parse_preamble(Parser & p, ostream & os,
 			if (name == "\\parindent" && content != "") {
 				if (content[0] == '0')
 					h_paragraph_separation = "skip";
-				else if (LYX_FORMAT >= 365)
-					h_paragraph_indentation = translate_len(content);
 				else
-					h_preamble << "\\setlength{" << name
-					           << "}{" << content << "}";
+					h_paragraph_indentation = translate_len(content);
 			} else if (name == "\\parskip") {
 				if (content == "\\smallskipamount")
 					h_defskip = "smallskip";
@@ -1085,10 +1221,18 @@ void parse_preamble(Parser & p, ostream & os,
 			string const color = p.getArg('{', '}');
 			string const space = p.getArg('{', '}');
 			string const value = p.getArg('{', '}');
-			if (LYX_FORMAT >= 382 &&
-			    color == "note_fontcolor" && space == "rgb") {
+			if (color == "document_fontcolor" && space == "rgb") {
+				RGBColor c(RGBColorFromLaTeX(value));
+				h_fontcolor = X11hexname(c);
+			} else if (color == "note_fontcolor" && space == "rgb") {
 				RGBColor c(RGBColorFromLaTeX(value));
 				h_notefontcolor = X11hexname(c);
+			} else if (color == "page_backgroundcolor" && space == "rgb") {
+				RGBColor c(RGBColorFromLaTeX(value));
+				h_backgroundcolor = X11hexname(c);
+			} else if (color == "shadecolor" && space == "rgb") {
+				RGBColor c(RGBColorFromLaTeX(value));
+				h_boxbgcolor = X11hexname(c);
 			} else {
 				h_preamble << "\\definecolor{" << color
 				           << "}{" << space << "}{" << value
@@ -1128,7 +1272,17 @@ void parse_preamble(Parser & p, ostream & os,
 			string const arg1 = p.verbatim_item();
 			string const arg2 = p.verbatim_item();
 			string const arg3 = p.verbatim_item();
-			if (!in_lyx_preamble) {
+			// test case \@ifundefined{date}{}{\date{}}
+			if (arg1 == "date" && arg2.empty() && arg3 == "\\date{}") {
+				h_suppress_date = "true";
+			// test for case
+			//\@ifundefined{showcaptionsetup}{}{%
+			// \PassOptionsToPackage{caption=false}{subfig}}
+			// that LyX uses for subfloats
+			} else if (arg1 == "showcaptionsetup" && arg2.empty()
+				&& arg3 == "%\n \\PassOptionsToPackage{caption=false}{subfig}") {
+				; // do nothing
+			} else if (!in_lyx_preamble) {
 				h_preamble << t.asInput()
 				           << '{' << arg1 << '}'
 				           << '{' << arg2 << '}'
@@ -1179,6 +1333,16 @@ string babel2lyx(string const & language)
 	return language;
 }
 
+
+/// translates a color name to a LyX color code
+string color2code(string const & name)
+{
+	char const * const * where = is_known(name, known_basic_colors);
+	if (where)
+		return known_basic_color_codes[where - known_basic_colors];
+	return name;
+}
+
 // }])
 
 
