@@ -48,11 +48,14 @@ void parse_preamble(Parser & p, std::ostream & os,
 	std::string const & forceclass, TeX2LyXDocClass & tc);
 /// Translate babel language name to LyX language name
 extern std::string babel2lyx(std::string const & language);
+/// translate color name to LyX color code
+extern std::string color2code(std::string const & name);
 
 /// used packages with options
 extern std::map<std::string, std::vector<std::string> > used_packages;
 extern const char * const modules_placeholder;
 extern std::string h_inputencoding;
+extern std::string h_paragraph_separation;
 
 /// in text.cpp
 std::string translate_len(std::string const &);
@@ -165,7 +168,7 @@ extern bool noweb_mode;
 /// Did we recognize any pdflatex-only construct?
 extern bool pdflatex;
 /// LyX format that is created by tex2lyx
-int const LYX_FORMAT = 345;
+int const LYX_FORMAT = 413;
 
 /// path of the master .tex file
 extern std::string getMasterFilePath();
